/*-----------------------------------------------------------------------

File  : ccl_grounding.h

Author: Stephan Schulz

Contents
 
  Definitions for functions (and possibly later data types)
  implementing grounding of near-propositional clause sets.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Tue May 29 02:20:15 CEST 2001
    New

-----------------------------------------------------------------------*/

#ifndef CCL_GROUNDING

#define CCL_GROUNDING

#include <ccl_clausesets.h>


/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

/* For a variable, keep track of possible and done instantiations */

typedef struct var_inst
{
   Term_p        variable;
   PStack_p      alternatives;
   PStackPointer position;
}VarInstCell, *VarInst_p;

/* For many variables, do the same */

typedef struct varset_inst
{
   long size;
   VarInst_p cells;
}VarSetInstCell, *VarSetInst_p;




/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define VarSetInstCellAlloc()    (VarSetInstCell*)SizeMalloc(sizeof(VarSetInstCell))
#define VarSetInstCellFree(junk) SizeFree(junk, sizeof(VarSetInstCell))

VarSetInst_p VarSetInstAlloc(Clause_p clause);
void         VarSetInstFree(VarSetInst_p junk);

bool EqnEqlitRecode(Eqn_p lit);
bool ClauseEqlitRecode(Clause_p clause);
long ClauseSetEqlitRecode(ClauseSet_p set);

long SigCountNonConstFSymbols(Sig_p sig);
long SigCollectGroundTerms(TB_p bank, PStack_p stack);

long ClauseCreateGroundInstances(TB_p bank, Clause_p clause,
				 VarSetInst_p inst, ClauseSet_p
				 groundset);

long ClauseSetCreateGroundInstances(TB_p bank, ClauseSet_p set,
				    ClauseSet_p groundset);

void ClausePrintDimacs(FILE* out, Clause_p clause);
void ClauseSetPrintDimacs(FILE* out, ClauseSet_p set, long max_literal);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





