/*-----------------------------------------------------------------------

File  : cco_signals.h

Author: Stephan Schulz

Contents
 
  Signal handler for limit signals...not really necessary, but may
  work around some Solaris bugs.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri Nov  6 14:50:28 MET 1998
    New

-----------------------------------------------------------------------*/

#ifndef CCO_SIGNALS

#define CCO_SIGNALS

#include <signal.h>
#include <cio_tempfile.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/




/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

/* rlim_t as used in Solaris is documented as "an integral type to
   which values of type int and long can be casted without information
   loss. At least in C-89, this implies that it is long. In glibc 2.0
   (and probably many traditional Unices), resource limits are of type
   int. Hence I use long as the most likely useful type compatible
   with both systems */

long                         SystemTimeLimit; /* Returned by the
						 initial getrlimit
						 call as general hard
						 time limit */
long                         SoftTimeLimit;   /* Try to exit
						 gracefully */
long                         HardTimeLimit;   /* Exit now! */
extern VOLATILE sig_atomic_t TimeIsUp;
extern VOLATILE sig_atomic_t TimeLimitIsSoft; /* Have we hit hard or
						 soft? */

void ESignalSetup(int mysignal);
void ESignalHandler(int mysignal);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





