#! /usr/bin/env python

import sys
import re
import string

white_space     = re.compile('\s+')
trail_space     = re.compile('\s*$')
full_comment    = re.compile('^#')
dash            = re.compile('-')
slash           = re.compile('/')
match_heuristic = re.compile("-H'\(.*\)'")
match_class     = re.compile('CLASS_[A-Z-]*$')
eval_f_sep      = re.compile('\),')

def parse_class(c, key, probs):
    # print key
    i=0
    p=open(c,'r')
    l=p.readline()[:-1]
    while l:
        probs[l] = key
        l=p.readline()[:-1]
        i=i+1
    p.close
    return i

def tuple_add2(t1,t2):
    return (t1[0]+t2[0],t1[1]+t2[1])

def parse_prot(filename, stratname, matrix):
    print "# Parsing " + stratname
    p=open(filename,'r')
    l=p.readline()[:-1]
    desc = "";
    while l:
        res = full_comment.match(l)
        if(res):
            desc = desc+l
        else:
            clean = re.sub(trail_space,'',l)                         
            tuple=re.split(white_space,clean)
            if tuple[-1]=="success":
                prob = tuple[0];
                cl=problems[prob]
                time=float(tuple[2])
                old = matrix[cl][stratname]
                matrix[cl][stratname]=tuple_add2(old,(1,time))
        l=p.readline()        
    p.close
    return desc

def eval_heuristic(classes,heuristic):
    res=(0.0,0.0)
    for i in classes:
        res=tuple_add2(res,matrix[i][heuristic])
    return res;

def tuple_is_better2(t1,t2):
    return t1[0]>t2[0] or (t1[0]==t2[0] and t1[0]<t1[1])

def find_optimal_heuristic(classes, exclude):
    res  = ""
    eval = (0.0,0.0)
    for i in stratset.keys():
        if not i in exclude:
            tmp = eval_heuristic(classes, i)
            if tuple_is_better2(tmp, eval):
                eval = tmp;
                res  = i
    return res

def find_covered(heuristic,classes):
    res = []
    for i in classes:
        if matrix[i][heuristic][0]==opt_res[i]:
            res.append(i)
    return res

def translate_class(cl):
    res = "      ( /* " + cl + " */\n"
    pref = "       ";
    for i in xrange(6,len(cl)):
        if cl[i]!="-":
            res = res+pref+"Spec"+class_int[i][cl[i]]+"(spec)"
            pref = "&&\n       "
    res = res+")"
    return res;

def translate_class_list(cl):
    res = "";
    pref = "";
    for i in cl:
        res = res+pref+translate_class(i)+"\n"
        pref = "       ||\n"
    return res[0:-1];

def trans_heuristic_name(name):
    return re.sub(dash,"_", name[10:])


def heuristic_define(name):
    mr = match_heuristic.search(stratdesc[name])
    if not mr:
        raise RuntimeError, "No heuristic defined in " + name;
    res= '"' + trans_heuristic_name(name) + ' = \\n"\n"'
    tmp = stratdesc[name][mr.start()+3:mr.end()-1]
    res=res+ re.sub(eval_f_sep,'),"\n" ',tmp) +'\\n"'    
    
    return res

class_int={
    6: {"U": "AxiomsAreUnit", "H": "AxiomsAreHorn", "G": "AxiomsAreGeneral"},
    7: {"U": "GoalsAreUnit", "H": "GoalsAreHorn", "G": "GoalsAreGeneral"},
    8: {"N": "NoEq", "S": "SomeEq", "P": "PureEq"},
    9: {"F": "FewNGPosUnits", "S": "SomeNGPosUnits", "M": "ManyNGPosUnits"},
    10:{"N": "GoalsHaveVars", "G": "GoalsAreGround"},
    11:{"F": "FewAxioms", "S": "SomeAxioms", "M": "ManyAxioms"},
    12:{"F": "FewLiterals", "S": "SomeLiterals", "M": "ManyLiterals"},
    13:{"S": "SmallTerms", "M": "MediumTerms", "L": "LargeTerms"},
    14:{"F": "FewGroundPos", "S": "SomeGroundPos", "M": "ManyGroundPos"}
}

selstrat={
   "NoSelection":"SelectNoLiterals",
   "NoGeneration":"SelectNoGeneration",
   "SelectNegativeLiterals":"SelectNegativeLiterals",
   "PSelectNegativeLiterals":"PSelectNegativeLiterals",
   "SelectPureVarNegLiterals":"SelectFirstVariableLiteral",
   "PSelectPureVarNegLiterals":"PSelectFirstVariableLiteral",
   "SelectLargestNegLit":"SelectLargestNegativeLiteral",
   "PSelectLargestNegLit":"PSelectLargestNegativeLiteral",
   "SelectSmallestNegLit":"SelectSmallestNegativeLiteral",
   "PSelectSmallestNegLit":"PSelectSmallestNegativeLiteral",
   "SelectLargestOrientable":"SelectLargestOrientableLiteral",
   "PSelectLargestOrientable":"PSelectLargestOrientableLiteral",
   "MSelectLargestOrientable":"MSelectLargestOrientableLiteral",
   "SelectSmallestOrientable":"SelectSmallestOrientableLiteral",
   "PSelectSmallestOrientable":"PSelectSmallestOrientableLiteral",
   "MSelectSmallestOrientable":"MSelectSmallestOrientableLiteral",
   "SelectDiffNegLit":"SelectDiffNegativeLiteral",
   "PSelectDiffNegLit":"PSelectDiffNegativeLiteral",
   "SelectGroundNegLit":"SelectGroundNegativeLiteral",
   "PSelectGroundNegLit":"PSelectGroundNegativeLiteral",
   "SelectOptimalLit":"SelectOptimalLiteral",
   "PSelectOptimalLit":"PSelectOptimalLiteral",
   "SelectOptimalLit":"SelectOptimalLiteral",
   "PSelectOptimalLit":"PSelectOptimalLiteral",
   "SelectCondOptimalLit":"SelectCondOptimalLiteral",
   "PSelectCondOptimalLit":"PSelectCondOptimalLiteral",
   "SelectAllCondOptimalLit":"SelectAllCondOptimalLiteral",
   "PSelectAllCondOptimalLit":"SelectAllPosCondOptimalLiteral",
   "SelectOptimalRestrDepth2":"SelectDepth2OptimalLiteral",
   "PSelectOptimalRestrDepth2":"PSelectDepth2OptimalLiteral",
   "SelectOptimalRestrPDepth2":"SelectPDepth2OptimalLiteral",
   "PSelectOptimalRestrPDepth2":"PSelectPDepth2OptimalLiteral",
   "SelectOptimalRestrNDepth2":"SelectNDepth2OptimalLiteral",
   "PSelectOptimalRestrNDepth2":"PSelectNDepth2OptimalLiteral",
   "SelectNonRROptimalLit":"SelectNonRROptimalLiteral",
   "PSelectNonRROptimalLit":"PSelectNonRROptimalLiteral",
   "SelectNonStrongRROptimalLit":"SelectNonStrongRROptimalLiteral",
   "PSelectNonStrongRROptimalLit":"PSelectNonStrongRROptimalLiteral",
   "SelectAntiRROptimalLit":"SelectAntiRROptimalLiteral",
   "PSelectAntiRROptimalLit":"PSelectAntiRROptimalLiteral",
   "SelectNonAntiRROptimalLit":"SelectNonAntiRROptimalLiteral",
   "PSelectNonAntiRROptimalLit":"PSelectNonAntiRROptimalLiteral",
   "SelectStrongRRNonRROptimalLit":"SelectStrongRRNonRROptimalLiteral",
   "PSelectStrongRRNonRROptimalLit":"PSelectStrongRRNonRROptimalLiteral",
   "SelectUnlessUniqMax":"SelectUnlessUniqMaxOptimalLiteral",
   "PSelectUnlessUniqMax":"PSelectUnlessUniqMaxOptimalLiteral",
   "SelectUnlessPosMax":"SelectUnlessPosMaxOptimalLiteral",
   "PSelectUnlessPosMax":"PSelectUnlessPosMaxOptimalLiteral",
   "SelectUnlessUniqPosMax":"SelectUnlessUniqPosMaxOptimalLiteral",
   "PSelectUnlessUniqPosMax":"PSelectUnlessUniqPosMaxOptimalLiteral",
   "SelectMinOptimalLit":"SelectMinOptimalLiteral",
   "PSelectMinOptimalLit":"PSelectMinOptimalLiteral",
   "SelectUnlessUniqMaxPos":"SelectUnlessUniqMaxPosOptimalLiteral",
   "PSelectUnlessUniqMaxPos":"PSelectUnlessUniqMaxPosOptimalLiteral",
   "SelectComplex":"SelectComplex",
   "PSelectComplex":"PSelectComplex",
   "SelectComplexExceptRRHorn":"SelectComplexExceptRRHorn",
   "PSelectComplexExceptRRHorn":"PSelectComplexExceptRRHorn",
   "SelectLComplex":"SelectLComplex",
   "PSelectLComplex":"PSelectLComplex",
   "SelectMaxLComplex":"SelectMaxLComplex",
   "PSelectMaxLComplex":"PSelectMaxLComplex",
   "SelectComplexPreferNEQ":"SelectComplexPreferNEQ",
   "PSelectComplexPreferNEQ":"PSelectComplexPreferNEQ",
   "SelectComplexPreferEQ":"SelectComplexPreferEQ",
   "PSelectComplexPreferEQ":"PSelectComplexPreferEQ",
   "SelectComplexExceptUniqMaxHorn":"SelectComplexExceptUniqMaxHorn",
   "PSelectComplexExceptUniqMaxHorn":"PSelectComplexExceptUniqMaxHorn",
   "MSelectComplexExceptUniqMaxHorn":"MSelectComplexExceptUniqMaxHorn"
}


argc = len(sys.argv)
i=1

problems   = {} # Keys are problems, values are problem classes
classlist  = [] # List of all class names
stratset   = {} # Names of strateges with associated file names
stratdesc  = {} # Names of strategies with associated command line 
matrix     = {} # Keys: Class names. Objects: Dictionaries associating
                # strategy with performance in this class

while i<argc:
    res = match_class.search(sys.argv[i])
    if(res):
        key = sys.argv[i][res.start():res.end()]
        classlist.append(key)
        parse_class(sys.argv[i], key, problems)
    else:
        key = re.split(slash,sys.argv[i])[-1]
        stratset[key] = sys.argv[i]
    i=i+1

for i in classlist:
    matrix[i] = {}
    for j in stratset.keys():
        matrix[i][j] = (0,0.0)

for i in stratset.keys():
    stratdesc[i]=parse_prot(stratset[i],i,matrix)    

sum=0
opt_res = {};
for i in classlist:
    solved = 0
    for j in matrix[i].keys():
        solved = max(solved, matrix[i][j][0])
    opt_res[i] = solved
    sum = sum+solved

class_list_iter = classlist
result = {}
used   = []

while class_list_iter:
    h = find_optimal_heuristic(class_list_iter, used)
    covered =  find_covered(h, class_list_iter)
    for i in covered:
        result[i] = h
        class_list_iter.remove(i)
    used.append(h)


# Do we want to optimize result? I.e. for each class pick the best (==
# fastest) of all picked stratgies?

by_heuristic={}

for i in result.keys():
    by_heuristic[result[i]]=[]

for i in result.keys():
    by_heuristic[result[i]].append(i)

print """
/* -------------------------------------------------------*/
/* The following code is generated automagically with     */
/* generate_auto.py. Yes, it is fairly ugly ;-)           */
/* -------------------------------------------------------*/

#ifdef CHE_PROOFCONTROL_INTERNAL

/* Strategies used:                                       */

"""

for i in by_heuristic.keys():
    print heuristic_define(i)

if used[0] in by_heuristic.keys():
    print "/* Global best, "+used[0]+", already defined */"
else:
    print "/* Global best (used as a default): */"
    print heuristic_define(used[0])

print """#endif

#if defined(CHE_HEURISTICS_INTERNAL) || defined(TO_ORDERING_INTERNAL)
"""

for i in by_heuristic.keys():
    print "   else if("
    print translate_class_list(by_heuristic[i])+")"
    print '''   {
#ifdef CHE_HEURISTICS_INTERNAL
      res = "''' + trans_heuristic_name(i) +'";'

    # Stuff for control
    print '''#endif
#ifdef TO_ORDERING_INTERNAL'''

    # Stuff for oparms
    print "#endif\n   }"

print "#endif"

print "\n/* Total solutions on test set:", sum, "*/"
print """/* -------------------------------------------------------*/
/*     End of automatically generated code.               */
/* -------------------------------------------------------*/
"""









