#!/usr/local/dist/bin/gawk -f

# Usage: interprete_cmd_line.awk <filename>
#
#   Given a protocol (with command line options in comment), return a
#   string containing C code to set the corresponding parameter in a
#   ProofControl-Object "control". This is only a development script
#   without error checking etc.
#
# Copyright 1999 Stephan Schulz, schulz@informatik.tu-muenchen.de

 

function print_strategy(to_print)
{
   print "   {";
   print "      res = \"" gensub(/-/, "_", "g", to_print)  "\";";
   
   selstrat = substr(to_print, length(to_print), 1);
   if(selstrat in strat)
   {
      print "      control->selection_strategy = " strat[selstrat] ";";	  
   }
   else
   {
      print "      control->selection_strategy = " strat["a"] ";";	
   }
   
   print "   }";
}

function get_cmd_line(file,   result)
{
   pipe =  "grep \"#.*-H'\\(.*\\)'\" " file;
   pipe | getline result;
   close(pipe);
   
   if(!result)
   {
      print "Command line must specify at least '-H' directive" > "/dev/stderr";
      exit 2;
   }
   return result;
}


function translate_cmd_line(string,   key, res)
{
   res = ""

   matcher = "-W[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      if(!selstrat[key])
      {
	 print "Error: Unknown selection strategy" > "/dev/stderr";
      }
      res = res "      control->strategy = " selstrat[key] ";\n";
   }
   matcher = "--literal-selection-strategy[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      if(!selstrat[key])
      {
	 print "Error: Unknown selection strategy" > "/dev/stderr";
      }
      res = res "      control->strategy = " selstrat[key] ";\n";
   }
   
   matcher = "-F[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);      
      res = res "      control->forward_demod = " key ";\n";
   }
   matcher = "--forward_demod_level=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->forward_demod = " key ";\n";
   }

   matcher = "--selection-pos-min=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->pos_lit_sel_min = " key ";\n";
   }
   matcher = "--selection-pos-max=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->pos_lit_sel_max = " key ";\n";
   }
   matcher = "--selection-neg-min=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->neg_lit_sel_min = " key ";\n";
   }
   matcher = "--selection-neg-max=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->neg_lit_sel_max = " key ";\n";
   }
    matcher = "--selection-all-min=[^ ]*";
    mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->all_lit_sel_min = " key ";\n";
   }
   matcher = "--selection-all-max=[^ ]*";
   mlen = index(matcher, "[") -1;
   if(match(string, matcher))
   { 
      key = substr(string, RSTART+mlen, RLENGTH-mlen);
      res = res "      control->all_lit_sel_max = " key ";\n";
   }  

   return res;
}

BEGIN{
   selstrat["NoSelection"]               = "SelectNoLiterals";
   selstrat["NoGeneration"]              = "SelectNoGeneration";   
   selstrat["SelectNegativeLiterals"]    = "SelectNegativeLiterals";          
   selstrat["PSelectNegativeLiterals"]   = "SelectPosNegativeLiterals";       
   selstrat["SelectPureVarNegLiterals"]  = "SelectFirstVariableLiteral";      
   selstrat["PSelectPureVarNegLiterals"] = "SelectPosFirstVariableLiteral";   
   selstrat["SelectLargestNegLit"]       = "SelectLargestNegativeLiteral";    
   selstrat["PSelectLargestNegLit"]      = "SelectPosLargestNegativeLiteral"; 
   selstrat["SelectSmallestNegLit"]      = "SelectSmallestNegativeLiteral";   
   selstrat["PSelectSmallestNegLit"]     = "SelectPosSmallestNegativeLiteral";
   selstrat["SelectDiffNegLit"]          = "SelectDiffNegativeLiteral";       
   selstrat["PSelectDiffNegLit"]         = "SelectPosDiffNegativeLiteral";    
   selstrat["SelectGroundNegLit"]        = "SelectGroundNegativeLiteral";     
   selstrat["PSelectGroundNegLit"]       = "SelectPosGroundNegativeLiteral";  
   selstrat["SelectOptimalLit"]          = "SelectOptimalLiteral";            
   selstrat["PSelectOptimalLit"]         = "SelectPosOptimalLiteral";         
   
   string = get_cmd_line(ARGV[1]);
   
   result = translate_cmd_line(string);
   
   print result;
   
   ARGV[1] = "";
}
  

   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   

   
   
   
   
   
   
   
   
   
   
   
   
   
